@@ -41,6 +41,7 @@ COMMANDS = {}
 # readers are introduced in later pandas.
 READERS = {
     "csv": lambda fname: pd.read_csv(fname),
+    "csv_with_sep": lambda fname, sep: pd.read_csv(fname, sep),
     "fwf": lambda fname: pd.read_fwf(fname),
     "json": lambda fname: pd.read_json(fname),
     "html": lambda fname: pd.read_html(fname),
@@ -221,8 +222,12 @@ def pipeout(df, sep=",", index=False, *args, **kwargs):
             pass
 
 
-def pipein(ftype="csv"):
+def pipein(ftype="csv", sep=None):
     try:
+        if sep is not None:
+            if ftype != "csv":
+                exit("Only csv mode handles separators")
+            return READERS["csv_with_sep"](sys.stdin, sep)
         return READERS[ftype](sys.stdin)
     except pd.errors.EmptyDataError:
         return pd.DataFrame()
@@ -394,6 +399,33 @@ def monotonic(column, direction="+"):
         print(df[column].is_monotonic_decreasing)
 
 
+@register
+def iplot(*args, **kwargs):
+    """Use plotly/cufflinks for interactive plot.
+
+    This option is similar to `plot` but creates an HTML file and opens a
+    browser for an interactive plot.
+
+    Usage: cat a.csv | ph iplot
+           cat a.csv | ph iplot --kind=bar
+           cat a.csv | ph iplot --kind=bar --barmode=stack
+
+
+    Depends on cufflinks: pip install ph[iplot].
+
+    """
+    try:
+        import cufflinks as cf
+        import plotly as py
+    except ImportError as err:
+        exit("iplot needs cufflinks, pip install ph[iplot]")
+
+    df = pipein()
+    fig = df.iplot(*args, asFigure=True, **kwargs)
+    py.offline.plot(fig)
+    pipeout(df)
+
+
 @register
 def plot(*args, **kwargs):
     """Plot the csv file.
@@ -454,23 +486,32 @@ def normalize(col=None):
 
 
 @register
-def date(col=None, unit=None, origin="unix"):
+def date(col=None, unit=None, origin="unix", errors="raise"):
     """Assemble datetime from multiple columns or from one column
 
     --unit can be D, s, us, ns (defaults to ns, ns from origin)
 
     --origin can be unix, julian, or time offset, e.g. '2000-01-01'
 
+    --errors can be raise, coerce, ignore (see pandas.to_datetime)
+
     Usage: cat a.csv | ph date x
            cat a.csv | ph date x --unit=s --origin="1984-05-17 09:30"
+           cat a.csv | ph date  # if a.csv contains year, month, date
 
     """
+    DATE_ERRORS = ("ignore", "raise", "coerce")
+    if errors not in DATE_ERRORS:
+        exit("Errors must be one of {}, not {}.".format(DATE_ERRORS, errors))
+
     df = pipein()
     try:
         if col is None:
-            df = pd.to_datetime(df)
+            df = pd.to_datetime(df, unit=unit, origin=origin, errors=errors)
         else:
-            df[col] = pd.to_datetime(df[col], unit=unit, origin=origin)
+            if col not in df.columns:
+                exit("No such column {}".format(col))
+            df[col] = pd.to_datetime(df[col], unit=unit, origin=origin, errors=errors)
     except Exception as err:
         exit(err)
 
@@ -503,7 +544,7 @@ def info():
 
 
 @register
-def to(ftype, fname=None):
+def to(ftype, fname=None, sep=None):
     """Export csv to given format (possibly csv).
 
     Supports csv, html, json, parquet, bigquery, tsv, etc. (see README for full
@@ -511,6 +552,7 @@ def to(ftype, fname=None):
 
     Usage: cat a.csv | ph to html
            cat a.csv | ph to tsv
+           cat a.csv | ph to csv --sep=';'
            cat a.csv | ph to clipboard
            cat a.csv | ph to json
            cat a.csv | ph to parquet out.parquet
@@ -540,12 +582,18 @@ def to(ftype, fname=None):
             print(content)
         exit()
 
+    if sep is not None:
+        if ftype != "csv":
+            exit("Only csv mode supports separator")
+
     writer = WRITERS[ftype]
     df = pipein()
     fn = getattr(df, writer)
     kwargs = {}
     if ftype == "tsv":
         kwargs["sep"] = "\t"
+    elif ftype == "csv" and sep is not None:
+        kwargs["sep"] = sep
     if fname is not None:
         print(fn(fname, **kwargs))
     else:
@@ -553,7 +601,7 @@ def to(ftype, fname=None):
 
 
 @registerx("from")
-def from_(ftype="csv"):
+def from_(ftype="csv", sep=None):
     """Read a certain (default csv) format from standard in and stream out as csv.
 
     Usage: cat a.json | ph from json
@@ -567,7 +615,13 @@ def from_(ftype="csv"):
     if ftype == "clipboard":
         pipeout(READERS["clipboard"]())
     else:
-        pipeout(pipein(ftype))
+        if sep is not None:
+            if ftype != "csv":
+                exit("Only csv mode accepts separator")
+            pipeout(pipein(ftype, sep=sep))
+
+        else:
+            pipeout(pipein(ftype))
 
 
 @register
@@ -696,7 +750,7 @@ def help(*args, **kwargs):
 
 
 @registerx("open")
-def open_(ftype, fname=None, sheet=0):
+def open_(ftype, fname=None, sheet=0, sep=None):
     """Use a reader to open a file.
 
     Open ftype file with name fname and stream out.
@@ -718,6 +772,9 @@ def open_(ftype, fname=None, sheet=0):
     kwargs = {}
     if ftype in ("excel", "xls", "odf"):
         kwargs["sheet"] = __tryparse(sheet)
+    if ftype == "csv" and sep is not None:
+        reader = READERS["csv_with_sep"]
+        kwargs["sep"] = sep
 
     if ftype == "clipboard" and fname is not None:
         exit("clipboard does not take fname")
@@ -733,6 +790,8 @@ def open_(ftype, fname=None, sheet=0):
         exit("{} is not supported in your Pandas installation\n{}".format(ftype, err))
     except ImportError as err:
         exit("{} is not supported in your Pandas installation\n{}".format(ftype, err))
+    except FileNotFoundError as err:
+        exit("File not found: {}".format(err))
     pipeout(df)
 
 
